var usersApi = {},
    common = require('./../../utils/common.js'),
    mail = require('./mail.js'),
	plugins = require('../../../plugins/pluginManager.js'),
    crypto = require('crypto');

(function (usersApi) {

    usersApi.getCurrentUser = function (params) {
        delete params.member.password;

        common.returnOutput(params, params.member);
        return true;
    };
    
    usersApi.getUserById = function (params) {
        if (!params.member.global_admin) {
            common.returnMessage(params, 401, 'User is not a global administrator');
            return false;
        }
        if (!params.qstring.id || params.qstring.id.length !== 24) {
            common.returnMessage(params, 401, 'Missing or incorrect user id parameter');
            return false;
        }
        common.db.collection('members').findOne({_id:common.db.ObjectID(params.qstring.id)},{password:0, appSortList:0}, function (err, member) {
    
            if (!member || err) {
                common.returnOutput(params, {});
                return false;
            }
    
            var memberObj = {};
    
            if(member.admin_of && member.admin_of.length > 0 && member.admin_of[0] == ""){
                member.admin_of.splice(0, 1);
            }
            if(member.user_of && member.user_of.length > 0 && member.user_of[0] == ""){
                member.user_of.splice(0, 1);
            }
            
            member['admin_of'] = ((member.admin_of && member.admin_of.length > 0) ? member.admin_of : []);
            member['user_of'] = ((member.user_of && member.user_of.length > 0) ? member.user_of : []);
            member['global_admin'] = (member.global_admin === true);
            member['locked'] = (member.locked === true);
            member['created_at'] = member.created_at || 0;
            member['last_login'] = member.last_login || 0;
            member['is_current_user'] = (member.api_key == params.member.api_key);
            memberObj[member._id] = member;
    
            common.returnOutput(params, memberObj);
        });
        return true;
    };

    usersApi.getAllUsers = function (params) {
        if (!params.member.global_admin) {
            common.returnMessage(params, 401, 'User is not a global administrator');
            return false;
        }
        common.db.collection('members').find({},{password:0, appSortList:0}).toArray(function (err, members) {
    
            if (!members || err) {
                common.returnOutput(params, {});
                return false;
            }
    
            var membersObj = {};
    
            for (var i = 0; i < members.length ;i++) {
                if(members[i].admin_of && members[i].admin_of.length > 0 && members[i].admin_of[0] == ""){
                    members[i].admin_of.splice(0, 1);
                }
                if(members[i].user_of && members[i].user_of.length > 0 && members[i].user_of[0] == ""){
                    members[i].user_of.splice(0, 1);
                }
               
                members[i]['admin_of'] = ((members[i].admin_of && members[i].admin_of.length > 0) ? members[i].admin_of : []);
                members[i]['user_of'] = ((members[i].user_of && members[i].user_of.length > 0) ? members[i].user_of : []);
                members[i]['global_admin'] = (members[i].global_admin === true);
                members[i]['locked'] = (members[i].locked === true);
                members[i]['created_at'] = members[i].created_at || 0;
                members[i]['last_login'] = members[i].last_login || 0;
                members[i]['is_current_user'] = (members[i].api_key == params.member.api_key);
                membersObj[members[i]._id] = members[i];
            }
    
            common.returnOutput(params, membersObj);
            return true;
        });
        return true;
    };

    usersApi.createUser = function (params) {
        if (!params.member.global_admin) {
            common.returnMessage(params, 401, 'User is not a global administrator');
            return false;
        }

        var argProps = {
                'full_name':    { 'required': true, 'type': 'String' },
                'username':     { 'required': true, 'type': 'String' },
                'password':     { 'required': true, 'type': 'String', 'min-length':plugins.getConfig("security").password_min, 'has-number': plugins.getConfig("security").password_number , 'has-upchar': plugins.getConfig("security").password_char, 'has-special': plugins.getConfig("security").password_symbol},
                'email':        { 'required': true, 'type': 'String' },
                'lang':         { 'required': false, 'type': 'String' },
                'admin_of':     { 'required': false, 'type': 'Array' },
                'user_of':      { 'required': false, 'type': 'Array' },
                'global_admin': { 'required': false, 'type': 'Boolean' }
            },
            newMember = {};

        if (!(newMember = common.validateArgs(params.qstring.args, argProps))) {
            common.returnMessage(params, 400, 'Not enough args');
            return false;
        }

        common.db.collection('members').findOne({ $or : [ {email: newMember.email}, {username: newMember.username} ] }, function(err, member) {
            if (member || err) {
                common.returnMessage(params, 200, 'Email or username already exists');
                return false;
            } else {
                createUser();
                return true;
            }
        });

        function createUser() {
            var passwordNoHash = newMember.password;
            newMember.password = common.sha512Hash(newMember.password);
            newMember.password_changed = 0;
            newMember.created_at = Math.floor(((new Date()).getTime()) / 1000); //TODO: Check if UTC
            newMember.admin_of = newMember.admin_of || [];
            newMember.user_of = newMember.user_of || [];
            newMember.locked = false;
            newMember.username = newMember.username.trim();
            newMember.email = newMember.email.trim();

            common.db.collection('members').insert(newMember, {safe: true}, function(err, member) {
                member = member.ops;
                if (member && member.length && !err) {

                    member[0].api_key = common.md5Hash(member[0]._id + (new Date().getTime()));
                    common.db.collection('members').update({'_id': member[0]._id}, {$set: {api_key: member[0].api_key}},function(){});

                    mail.sendToNewMember(member[0], passwordNoHash);
                    plugins.dispatch("/i/users/create", {params:params, data:member[0]});
                    delete member[0].password;

                    common.returnOutput(params, member[0]);
                } else {
                    common.returnMessage(params, 500, 'Error creating user');
                }
            });
        }

        return true;
    };

    usersApi.updateUser = function (params) {
        var argProps = {
                'user_id':      { 'required': true, 'type': 'String', 'min-length': 24, 'max-length': 24, 'exclude-from-ret-obj': true },
                'full_name':    { 'required': false, 'type': 'String' },
                'username':     { 'required': false, 'type': 'String' },
                'password':     { 'required': false, 'type': 'String', 'min-length':plugins.getConfig("security").password_min, 'has-number': plugins.getConfig("security").password_number , 'has-upchar': plugins.getConfig("security").password_char, 'has-special': plugins.getConfig("security").password_symbol},
                'email':        { 'required': false, 'type': 'String' },
                'lang':         { 'required': false, 'type': 'String' },
                'admin_of':     { 'required': false, 'type': 'Array' },
                'user_of':      { 'required': false, 'type': 'Array' },
                'global_admin': { 'required': false, 'type': 'Boolean' },
                'locked':       { 'required': false, 'type': 'Boolean' },
                'send_notification': { 'required': false, 'type': 'Boolean', 'exclude-from-ret-obj': true }
            },
            updatedMember = {},
            passwordNoHash = "";

        if (!(updatedMember = common.validateArgs(params.qstring.args, argProps))) {
            common.returnMessage(params, 400, 'Not enough args');
            return false;
        }

        if (!(params.member.global_admin || params.member._id === params.qstring.args.user_id)) {
            common.returnMessage(params, 401, 'User is not a global administrator');
            return false;
        }

        if (updatedMember.password) {
            passwordNoHash = updatedMember.password;
            updatedMember.password = common.sha512Hash(updatedMember.password);
            if(params.member._id !== params.qstring.args.user_id){
                updatedMember.password_changed = 0;
            }
        }
        if (updatedMember.username) {
            updatedMember.username = updatedMember.username.trim();
        }
        if (updatedMember.email) {
            updatedMember.email = updatedMember.email.trim();
        }
        function killAllSessionForUser(userId) {
            common.db.collection('sessions_').find({
                "session": { $regex: userId }
            }).toArray(function (err, sessions) {

                sessions.map(function (currentSession) {
                    if (currentSession.session && JSON.parse(currentSession.session) && JSON.parse(currentSession.session).uid === userId) {
                        var sessionObject = JSON.parse(currentSession.session);

                        delete sessionObject.fails;
                        delete sessionObject.uid;
                        delete sessionObject.gadm;
                        delete sessionObject.email;
                        delete sessionObject.expires;

                        common.db.collection('sessions_').update({
                            "_id": currentSession._id
                        }, {
                                "$set": {
                                    session: JSON.stringify(sessionObject)
                                }
                            });
                    }
                })
            })

        }
        common.db.collection('members').findOne({'_id': common.db.ObjectID(params.qstring.args.user_id)}, function(err, memberBefore) {
            common.db.collection('members').update({'_id': common.db.ObjectID(params.qstring.args.user_id)}, {'$set': updatedMember}, {safe: true}, function(err, isOk) {
                common.db.collection('members').findOne({'_id': common.db.ObjectID(params.qstring.args.user_id)}, function(err, member) {
                    if (member && !err) {
                        updatedMember._id = params.qstring.args.user_id;
                        plugins.dispatch("/i/users/update", {params:params, data:updatedMember, member:memberBefore});
                        if (params.qstring.args.send_notification && passwordNoHash) {
                            mail.sendToUpdatedMember(member, passwordNoHash);
                        }

                        if (updatedMember.password && params.member._id.toString() !== params.qstring.args.user_id.toString()) {
                            killAllSessionForUser(updatedMember._id)
                        }

                        common.returnMessage(params, 200, 'Success');
                    } else {
                        common.returnMessage(params, 500, 'Error updating user');
                    }
                });
            });
        });

        return true;
    };

    usersApi.deleteUser = function (params) {
        var argProps = {
                'user_ids': { 'required': true, 'type': 'Array' }
            },
            userIds = [];

        if (!params.member.global_admin) {
            common.returnMessage(params, 401, 'User is not a global administrator');
            return false;
        }

        if (!(userIds = common.validateArgs(params.qstring.args, argProps).user_ids)) {
            common.returnMessage(params, 400, 'Not enough args');
            return false;
        }

        for (var i = 0; i < userIds.length; i++) {
            // Each user id should be 24 chars long and a user can't delete his own account
            if (!userIds[i] || userIds[i] === params.member._id + "" || userIds[i].length !== 24) {
                continue;
            } else {
				common.db.collection('members').findAndModify({'_id': common.db.ObjectID(userIds[i])},{},{},{remove:true},function(err, user){
					if(!err && user && user.ok)
						plugins.dispatch("/i/users/delete", {params:params, data:user.value});
				});
            }
        }

        common.returnMessage(params, 200, 'Success');
        return true;
    };

}(usersApi));

module.exports = usersApi;